/*
 * Copyright (c) 2017 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution  and  use  in  source  and binary  forms,  with  or  without
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 * THE SOFTWARE  IS PROVIDED "AS IS"  AND THE AUTHOR  DISCLAIMS ALL WARRANTIES
 * WITH  REGARD   TO  THIS  SOFTWARE  INCLUDING  ALL   IMPLIED  WARRANTIES  OF
 * MERCHANTABILITY AND  FITNESS.  IN NO EVENT  SHALL THE AUTHOR  BE LIABLE FOR
 * ANY  SPECIAL, DIRECT,  INDIRECT, OR  CONSEQUENTIAL DAMAGES  OR  ANY DAMAGES
 * WHATSOEVER  RESULTING FROM  LOSS OF  USE, DATA  OR PROFITS,  WHETHER  IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR  OTHER TORTIOUS ACTION, ARISING OUT OF OR
 * IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 *                                           Anthony Mallet on Mon Feb 27 2017
 */
#include "acheader.h"

#include <avr/interrupt.h>
#include <avr/pgmspace.h>

#include "arduio.h"

int
main()
{
  struct ario_msg *msg;

  /* configure pins */
  ario_pin_init();

  /* serial, 2Mbps */
  ario_uart_init(2000000);

  /* process messages */
  sei();
  while(1) {

    while ((msg = ario_recv())) {
      uint8_t cmd = msg->data[0];

      switch(cmd) {
        case 'd': /* GPIO direction */
          if (msg->len == 3) {
            ario_dpin_dir(msg->data[1], msg->data[2]);
          }
          ario_ddir_state();
          break;

        case 'o': /* GPIO out */
          if (msg->len == 3) {
            ario_dpin_out(msg->data[1], msg->data[2]);
          }
          break;

        case 'O': /* GPIO out */
          if (msg->len > 1) {
            uint8_t i;
            for(i = 1; i < msg->len; i++) {
              ario_dpin_out(i - 1, msg->data[i]);
            }
          }
          break;

        case 'i': /* GPIO in */
          if (msg->len == 2) {
            ario_log(PSTR("i%1%1"), msg->data[1], ario_dpin_in(msg->data[1]));
          }
          break;

        case 'a': /* analog in */
          if (msg->len == 2) {
            ario_log(PSTR("a%1%1"), msg->data[1], ario_apin_in(msg->data[1]));
          }
          break;

        case 's': /* full state */
          if (msg->len == 1) {
            ario_dpin_state();
            ario_apin_state();
          }
          break;

        case '?': /* id */
          if (msg->len == 1) {
            ario_log(PSTR("?arduio" PACKAGE_VERSION));
          }
          break;
      }
    }

    ario_send();
  }
}
